#include <stdio.h>
#include "apollo.h"

#define USAGE 	"apollo - ver 0.0.0 2004/02/17 \n"\
			  	" trip ver 0.2(apollo512-2) 2004/02/16\n\n"\
			  	" -g  Generate KeyPair\n"\
			  	"      input(stdin)  : KeyGenerator\n"\
			  	"      output(stdout): PublicKeyStr\n"\
			  	"                      SecretKeyStr\n\n"\
			  	" -s  Sign Message\n"\
			  	"      input(stdin)  : SignTarget\n"\
			  	"                      PublicKeyStr\n"\
			  	"                      SecretKeyStr\n"\
			  	"      output(stdout): SignatureStr\n\n"\
			  	" -v  Verify Signature\n"\
			  	"      input(stdin)  : SignTarget\n"\
			  	"                      SuspectSignature\n"\
			  	"                      PublicKeyStr\n"\
			  	"      output(stdout): True / False\n\n"\
			  	" -c  Cut KeyStr to 11words\n"\
			  	"      input(stdin)  : KeyStr\n"\
			  	"      output(stdout): ShortKeyStr\n"\

static void test(void);

int main(int argc,char *argv[])
{
	char buf1[512];
	char buf2[512];
	char buf3[512];
	char buf4[512];
	int len1,len2,len3;
	
	if(argc < 2){
		printf(USAGE);
		return 0;
	}
	if((argv[1][0] == '-') || argv[1][0] == '/'){
		switch(argv[1][1]){
			case 'g':
			case 'G':
				fgets(buf1,511,stdin);
				buf1[511] = '\0';
				len1 = strlen(buf1);
				if (buf1[len1-1] == '\n')
					buf1[--len1] = '\0';
				
				keycreate512(buf2,buf3,buf1,len1);
				puts(buf2);
				puts(buf3);
				return 0;
			
			case 's':
			case 'S':
				fgets(buf1,511,stdin);
				buf1[511] = '\0';
				len1 = strlen(buf1);
				if (buf1[len1-1] == '\n')
					buf1[--len1] = '\0';
				fgets(buf2,511,stdin);
				buf2[511] = '\0';
				len2 = strlen(buf2);
				if (buf2[len2-1] == '\n')
					buf2[--len2] = '\0';
				fgets(buf3,511,stdin);
				buf3[511] = '\0';
				len3 = strlen(buf3);
				if (buf3[len3-1] == '\n')
					buf3[--len3] = '\0';
				
				RSAsign(buf4,buf1,len1,buf2,buf3,len2);
				puts(buf4);
				return 0;
				
			case 'v':
			case 'V':
				fgets(buf1,511,stdin);
				buf1[511] = '\0';
				len1 = strlen(buf1);
				if (buf1[len1-1] == '\n')
					buf1[--len1] = '\0';
				fgets(buf2,511,stdin);
				buf2[511] = '\0';
				len2 = strlen(buf2);
				if (buf2[len2-1] == '\n')
					buf2[--len2] = '\0';
				fgets(buf3,511,stdin);
				buf3[511] = '\0';
				len3 = strlen(buf3);
				if (buf3[len3-1] == '\n')
					buf3[--len3] = '\0';
				
				if(RSAverify(buf1,len1,buf2,buf3,len3) == 0)
					puts("True");
				else
					puts("False");
				return 0;
			
			case 'c':
			case 'C':
				fgets(buf1,511,stdin);
				buf1[511] = '\0';
				len1 = strlen(buf1);
				if (buf1[len1-1] == '\n')
					buf1[--len1] = '\0';
				
				triphash(buf2,buf1,len1);
				puts(buf2);
				return 0;
				
			case 't':
				test();
				return 0;
		}
	}
	
	
	printf(USAGE);
	return 0;
}

char *testseeds[] = {"test","hoge","Winny","V","Crescent","Download",
					"47","((( ;߄D))))޸޸ ","(߁)!!!!! ",
					"3l"};
char *testhashs[] = {"12345678901234567890123456789012","aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
					"͉ĂĂ܂Ȃw ","nbV̕񂪓̏ꏊ"};

static void test(void)
{
	char buf1[512];
	char buf2[512];
	char buf3[512];
	char buf4[512];
	int len1,len2,len3;
	int i,j;
	
	printf("test mode: 10keys * 4hashs\n");
	for(i = 0;i < 10;i++){
		len1 = strlen(testseeds[i]);
		keycreate512(buf2,buf3,testseeds[i],len1);
		printf("\nKeyGenerator: %s\n",testseeds[i]);
		printf("PublicKey = %s\n",buf2);
		printf("SecretKey = %s\n",buf3);
		len2 = strlen(buf2);
		triphash(buf1,buf2,len2);
		printf("Trip = %s\n\n",buf1);
		
		for(j = 0;j < 4;j++){
			len3 = strlen(testhashs[j]);
			RSAsign(buf4,testhashs[j],len3,buf2,buf3,len2);
			printf(" hash = %s\n",testhashs[j]);
			printf(" Signature = %s\n",buf4);
			if(RSAverify(testhashs[j],len3,buf4,buf2,len2) == 0)
				printf("  True\n");
			else
				printf("  False\n");
			buf4[0] = '*';
			printf(" BrokenSignature = %s\n",buf4);
			if(RSAverify(testhashs[j],len3,buf4,buf2,len2) == 0)
				printf("  True\n");
			else
				printf("  False\n");
		}
	}
}
